// Copyright (C) 2020 VIRNECT CO., LTD.
// All rights reserved.

#pragma once

#include "Shader.hpp"

#include <VARAPI/TrackingFramework/TrackingFramework.hpp>
#include <glad/glad.h>

/**
 * @brief Background Texture rendering class
 */
class BackgroundTexture
{
public:
    /**
     * Initialize function for create texture
     * @param width Camera frame's width
     * @param height Camera frame's height
     */
    BackgroundTexture(uint32_t width, uint32_t height);

    /**
     * @brief Destruct function for release texture instance
     */
    ~BackgroundTexture();

    /**
     * @brief Draw Background camera texture
     * @param img VIRNECT Image
     */
    void render(const VIRNECT::ImageRGB& img);

private:
    const char* vertexShaderSource = "#version 330 core\n"
                                     "layout (location = 0) in vec3 aPos;\n"
                                     "layout (location = 1) in vec2 aTexCoord;\n"
                                     "out vec4 ourColor;\n"
                                     "out vec2 TexCoord;\n"
                                     "void main()\n"
                                     "{\n"
                                     "   gl_Position = vec4(aPos, 1.0);\n"
                                     "   TexCoord = vec2(aTexCoord.x, aTexCoord.y);\n"
                                     "}\0"; ///< Background texture's vertex shader code
    const char* fragmentShaderSource = "#version 330 core\n"
                                       "out vec4 FragColor;\n"
                                       "in vec2 TexCoord;\n"
                                       "uniform sampler2D texture1;\n"
                                       "void main()\n"
                                       "{\n"
                                       "   FragColor = texture(texture1, TexCoord);\n"
                                       "}\n\0"; ///< Background texture's fragment shader code

    uint32_t mWidth = 0;                                                         ///< Texture width
    uint32_t mHeight = 0;                                                        ///< Texture height
    GLuint mTextureID = 0;                                                       ///< Texture ID
    Shader mBackgroundShader = Shader(vertexShaderSource, fragmentShaderSource); ///< Shader instance which has shader codes
    uint32_t mVBO = 0;                                                           ///< Vertex Buffer Object ID
    uint32_t mVAO = 0;                                                           ///< Vertex Array Object ID
    uint32_t mEBO = 0;                                                           ///< Element Buffer Object ID
};